#!/usr/bin/env python
# File created on 2016/4/6
__author__ = "Li,Chaonan"
__copyright__ = "Copyright 2016, Li,Xiangzhen Group, CIB, Chinese Academy of Sciences"
__license__ = "GPL"
__version__ = "1.1"
__maintainer__ = "Li,Xiangzhen Group"
__email__ = "chaonanl@yahoo.com"

from optparse import OptionParser
from Bio import Entrez
from Bio import SeqIO
import re
import os

#make options 
def MakeOption():
	parser=OptionParser(usage="%prog [-h] [-v] -i[--input=] -d[--protein_ids=] -o[--output=]\n\nDescription:\n \
This script is designed to download amino acid sequences from GenBank. Users must ensure that the computer has connetd to network.\
The process of downloading may take a long time, so we recommend users use Work Station when run this script.\n\n\
INFORMATION:\n\
Author: Li, Chaonan\n\
Copyright: Copyright 2016, Li,Xiangzhen Group, CIB, Chinese Academy of Sciences\n\n", version="%prog 1.1")
	parser.add_option("-i", "--input", action="store", dest="inputfile",  
                      help="The DNA alignment database provided Marc et al in 2013 with tab-seperated text format [input file].", 
                      default=False)
	parser.add_option("-d", "--protein_ids", action="store", dest="proteinID",  
                      help="The accession number of DNA and GI number of protein sequences returnd from Genbank with tab-seperated format [output file]", 
                      default=False)
	parser.add_option("-o", "--output", action="store", dest="outputfile",  
                      help="The outputfile saving amino acid sequences with fasta format [output file]", 
                      default=False)
					  
	(options, args) = parser.parse_args()
	input=options.inputfile # the input file with fasta format
	gi_ac=options.proteinID# we will save the accession numbers and GI numbers of each amino acid sequences into a local file
	output=options.outputfile# the outputfile saving amino acid sequences with fasta format
	return(input,gi_ac,output)

# This function used to download amino acid online
def ID_download(inputs,gi_acs):
	print "Query amino acid accession numbers and GI numbers from Genbank... "
	protein_gi_ac=open(gi_acs,"w")
	ac_list=[]
	for each in SeqIO.parse(inputs,"fasta"):
		ac=each.id
		seqs=each.seq
		ac_list.append(ac)
	a=0
	b=0
	for eachs in ac_list:
		if "-" in eachs:
			pass
		elif "_" in eachs:
			pass
		else:
			Entrez.email="A.N.Other@example.com"
			handle=Entrez.efetch(db="nucleotide",id=str(eachs),rettype="gb",retmode="text")
			files=handle.read()
			patt1='/protein_id="(.*).*'
			patt2='/db_xref="GI:(.*).*'
			if len(re.findall(patt1,files)) and len(re.findall(patt2,files)):
				a+=1
				prot_ac=re.findall(patt1,files)[0].strip('"')
				prot_gi=re.findall(patt2,files)[0].strip('"')
				print "query id: %s"%eachs
				print '...downloaded %s records...    GI:%s    AC:%s'%(a,prot_gi,prot_ac)
				protein_gi_ac.write("%s\t%s\t%s\n"%(eachs,prot_gi,prot_ac))
			else:
				b+=1
				pass
	protein_gi_ac.close()
	print "matched id: %s"%a
	print "can not match id: %s"%b

# This function is used to download amino acid sequence from Genbank
def Seq_download(gi_acs,outputs):
	output=open(outputs,"w")
	a=0
	for eachline in open(gi_acs):
		eachline_list=eachline.strip("\n").split("\t")
		gi=eachline_list[1]
		Entrez.email="A.N.Other@example.com"
		handle=Entrez.efetch(db="protein",id=str(gi),rettype="fasta",retmode="text")
		files=handle.read()
		output.write(files)
		a+=1
		print "...Downloaded %s records..."%a
	output.close()

def main():
	file=MakeOption()
	INPUT=file[0]
	GI_AC=file[1]
	OUTPUT=file[2]
	ID_download(INPUT,GI_AC)
	Seq_download(GI_AC,OUTPUT)
if __name__ == "__main__":
	main()
